@@ -1,8 +1,24 @@
 
 """
-  Copyright (c) 2014, SunSpec Alliance
-  All Rights Reserved
+    Copyright (C) 2017 SunSpec Alliance
 
+    Permission is hereby granted, free of charge, to any person obtaining a
+    copy of this software and associated documentation files (the "Software"),
+    to deal in the Software without restriction, including without limitation
+    the rights to use, copy, modify, merge, publish, distribute, sublicense,
+    and/or sell copies of the Software, and to permit persons to whom the
+    Software is furnished to do so, subject to the following conditions:
+
+    The above copyright notice and this permission notice shall be included
+    in all copies or substantial portions of the Software.
+
+    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+    IN THE SOFTWARE.
 """
 
 import os
@@ -115,7 +131,7 @@ class ClientDevice(device.Device):
                 if delay is not None:
                     time.sleep(delay)
 
-        if self.base_addr:
+        if self.base_addr is not None:
             # print 'base address = %s' % (self.base_addr)
             model_id = util.data_to_u16(data[4:6])
             addr = self.base_addr + 2
@@ -194,7 +210,7 @@ class ClientModel(device.Model):
                     # print 'data len = ', len(data)
                     data_len = len(data)/2
                     if data_len != self.len:
-                        raise SunSpecClientError('Error reading model %s' % model_type)
+                        raise SunSpecClientError('Error reading model %s' % self.model_type)
 
                     #  for each repeating block
                     for block in self.blocks:
@@ -235,11 +251,30 @@ class ClientModel(device.Model):
 
     def write_points(self):
 
+        addr = None
+        next_addr = None
+        data = ''
+
         for block in self.blocks:
             for point in block.points_list:
                 if point.dirty:
-                    # print 'Updating %s' % (point.point_type.id)
-                    point.write()
+                    point_addr = int(point.addr)
+                    point_len = int(point.point_type.len)
+                    point_data = point.point_type.to_data(point.value_base, (point_len * 2))
+                    if addr is None:
+                        addr = point_addr
+                        data = ''
+                    else:
+                        if point_addr != next_addr:
+                            block.model.device.write(addr, data)
+                            addr = point_addr
+                            data = ''
+                    next_addr = point_addr + point_len
+                    data += point_data
+                    point.dirty = False
+            if addr is not None:
+                block.model.device.write(addr, data)
+                addr = None
 
 class ClientBlock(device.Block):
 
@@ -257,6 +292,7 @@ class ClientPoint(device.Point):
 
         data = self.point_type.to_data(self.value_base, (int(self.point_type.len) * 2))
         self.block.model.device.write(int(self.addr), data)
+        self.dirty = False
 
 class SunSpecClientModelBase(object):
 
